/*
 * PWM_generator.c
 *
 * Created: 30.09.2014 10:31:11
 *  Author: Vegard
 */ 

#define PWMPIN PB0
#define INPUTPIN PB1
#define F_CPU 8000000

#include <avr/io.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include <stdlib.h>


// Routines to set and clear bits (used in the sleep code)
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

int makeSignal = 0;

ISR(PCINT0_vect)	     // interrupt service routine
{				
	//Change interupt flag
	makeSignal = 1;
	return;
}

void setup_watchdog(int ii) {

	int bb;
	//int ww;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	//ww=bb;

	MCUSR &= ~(1<<WDRF);
	// start timed sequence
	WDTCR |= (1<<WDCE) | (1<<WDE);
	// set new watchdog timeout value
	WDTCR = bb;
	WDTCR |= _BV(WDIE);
}

void sleep(){
	// sleep mode is set here
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); 
	sleep_enable();
	
	// System actually sleeps here
	sleep_mode();                        
	
	// System continues execution here when watchdog timed out
	sleep_disable();                     
}


void PWM_init(){
	//Enable input on the inputpin
	DDRB = 0x00;
	
	//Set the inputpin low
	PORTB |= (0 << INPUTPIN);
	
	// pin change mask: listen to portb bit 2
	PCMSK |= (1<<PCINT1);	
	
	// enable PCINT interrupt
	GIMSK |= (1<<PCIE);	
	
	// enable all interrupts
	sei();			
	
	//Enable output on the outputpin
	DDRB |= (1 << PWMPIN);
	
	//Clear OC0A on Compare Match , using fast PWM mode
	TCCR0A |= (1 << COM0A1) | (1 << WGM01) | (1 << WGM00);
	
	//Fast PWM mode with 0xFF = TOP, Clock select 64 prescaler
	TCCR0B |= (0 << WGM02) | (1 << CS01) | (1 << CS00);
	
	//Init the PWM signal to be compared with the counter value
	OCR0A = 0;
	
	// approximately xxx seconds sleep
	setup_watchdog(2); 
}

void PWM_run(){
	uint64_t value;
	int mini=0;
	for(value = mini; value < 50; value++){
		OCR0A = value;
		_delay_ms(2);
	}
	
	for(value = 50; value > mini; value --){
		OCR0A = value;
		_delay_ms(2);
	}
}

int main(void)
{
	PWM_init();
		
    while(1)
    {
         if(makeSignal == 1){
			 PWM_run();
			 makeSignal = 0;
		 }
		 else{
			 OCR0A = 0;
			 DDRB |= (0 << PWMPIN);
			 sleep();
			 DDRB |= (1 << PWMPIN);
		 }
    }
}


//SIRIS:
/*
#include <avr/io.h>
#define PWMPIN PB0
#define INPUTPIN PB1
void PWM_init(){
	//Enable input on the inputpin
	DDRB = 0x00;
	//Set the inputpin low
	PORTB |= (0 << INPUTPIN);
	//Enable output on the outputpin
	DDRB |= (1 << PWMPIN);
	//Clear OC0A on Compare Match , using fast PWM mode
	TCCR0A |= (1 << COM0A1) | (1 << WGM01) | (1 << WGM00);
	//Fast PWM mode with 0xFF = TOP, Clock selecy 64 prescaler
	TCCR0B |= (0 << WGM02) | (1 << CS01) | (1 << CS00);
	//Init the PWM signal to be compared with the counter value
	OCR0A = 0;
}
void PWM_run(){
	uint64_t value;
	uint64_t sleeper;
	for(value = 0; value < 50; value++){
		OCR0A = value;
		sleeper = 0;
		while(sleeper < 300){
			sleeper++;
		}
	}
	for(value = 50; value > 0; value --){
		OCR0A = value;
		sleeper = 0;
		while(sleeper < 300){
			sleeper++;
		}
	}
}
int main(void)
{
	PWM_init();
	while(1)
	{
		if(PINB & (1 << INPUTPIN)){
			PWM_run();
		}else{
			OCR0A = 0;
		}
	}
}*/