 clear all;
 close all;
 clc;
%%
% Define constants
iter = 10;
% Physical properties of object
A = pi*0.05^2; % Cross-sectional area of a sphere
C_D = 0.47; % Drag constant for a sphere
m = 0.2; % Mass of the object
rho = 1.225; % Air density at +15 degrees Celsius
b = 0.5*A*C_D*rho;
g = 9.81;
% Wind velocity
w = 5;
w_angle = 180;
v_w = [w*cos(w_angle);w*sin(w_angle);0];
w_uncertainty = 0.5;
w_angle_uncert = 10;
w_magn_range = (w*(1-w_uncertainty)):w*w_uncertainty/iter:(w*(1+w_uncertainty));
w_angle_range = (w_angle-w_angle_uncert):(w_angle_uncert/iter):(w_angle+w_angle_uncert);
% Initial velocity
v_init = 28;
% Initial position
x_0_bx = 0;
x_0_by = 0;
x_0_alt = 50;
mu_pos = [pi x_0_by];
bx_uncert = 5;
by_uncert = 5;
bx_range = 0:bx_uncert/iter/2:bx_uncert;
by_range = -by_uncert:by_uncert/iter:by_uncert;
[bX1,bX2] = meshgrid(bx_range, by_range);
sigma_pos = [1 0; 0 1];
norm_pos = mvnpdf([bX1(:) bX2(:)], mu_pos, sigma_pos);
norm_pos = reshape(norm_pos,length(bx_range), length(by_range));
figure
surf(bx_range, by_range, norm_pos);
% Simulation time
dt = 0.01;
tspan = 0:dt:6;
% Initialize ground level matrix 1000x1000 m
Impact = zeros(1000,1000);
% Iterations
tot_iter = length(w_magn_range)*length(w_angle_range);
i_ = 1;
% Create waitbar to show progress
h = waitbar(0,'wait...');
% Simulate position given varying wind
for i=1:(length(w_magn_range))
    for j=1:(length(w_angle_range))
        
        w = w_magn_range(i);
        w_angle = w_angle_range(j);
        w_angle = w_angle/180*pi;
        v_w = [w*cos(w_angle);w*sin(w_angle);0];
        
        v_0 = [v_init+w*cos(w_angle);v_init+w*sin(w_angle);0];
        x_0 = [v_0;x_0_bx;x_0_by;x_0_alt];

        % Solve differential system
        [t,v] = ode45(@(t,v) freeFallFunc2(t,v,v_w,b,m,g), tspan, x_0);
        figure(98)
        plot3(v(:,4), v(:,5),v(:,6))
        hold on;grid on;xlabel('x');ylabel('y');zlabel('z');
        % Position
        x_incr_reso = v(end,4:6)*1;
        x_rounded = round(x_incr_reso(:));
        Impact(x_rounded(1),x_rounded(2)) =+ norm_pos(i,j);
        % Update waitbar
        i_ = i_+1;
        waitbar(i_/tot_iter,h);
    end
end
close(h);
x_start = 35;
y_start = x_start;
x_end = 55;
y_end = x_end;
points = 20;
figure
surf(linspace(x_start,x_end,points),linspace(y_start,...
    y_end,points),Impact(x_start:x_end-1,y_start:y_end-1))
